

set -ex



ls -A1 ${PREFIX}/bin/*
ls -A1 ${PREFIX}/include/*
ls -A1 ${PREFIX}/lib/*
icpx --dpcpp -fsycl --gcc-toolchain=$PREFIX --sysroot=$PREFIX/$HOST/sysroot -target $HOST ${LDFLAGS} ${CXXFLAGS} simple.cpp -lpthread -o simple
exit 0
